#! /bin/bash

# Author : San Bergmans
# Date   : 11-1-2015
# Origin : www.sbprojects.net

# This script will add two lines to the crontab, or alter them if
# they already exist. One line is set at sunrise ( ± offset), the
# other is set at sunset ( ± offset). 

# Installation

# Copy this script in your ~/bin directory
# Change, if necessary, these variables:
# BINPATH		-->	Path to the binary path which contains this script
# 					and the sunrise and sunset scripts.
# OFFSET_RISE	--> Offset from sunrise (max ± 60 minutes)
# OFFSET_SET	--> Offset from sunset (max ± 60 minutes)
# CRON_CMD_RISE --> Command to be executed at sunrise
# CRON_CMD_SET  --> Command to be executed at sunset
# latitude		--> In Perl script. Your location. Positive is East
# longitude		--> In Perl script. Your longitude. Positive is North

# Then simply run this script manually once. From then on it will run
# daily just after midnight to set today's sunrise and sunset times.

# Some path information
BINPATH="$HOME/bin"
TEMP_FILE="/tmp/cronsunriset"

# Configuration
OFFSET_RISE=0		# Offset in minutes to sunrise (max ± 60 min)
OFFSET_SET=0		# Offset in minutes to sunset (max ± 60 min)

# Scripts to run at sunrise and sunset
CRON_CMD_RISE="$BINPATH/loff 2>&1"
CRON_CMD_SET="$BINPATH/lon 2>&1"

# Crontab extras
CRON_HEADER="# Sunrise and sunset scripts"
CRON_FLG_RISE="# cron at sunrise"
CRON_FLG_SET="# cron at sunset"

# Main cron task which sets the switching times for the next day
CRON_TSK_FLG="# Set new sunrise and sunset times"
CRON_TSK_CMD="01\t0\t*\t*\t*\t$BINPATH/cronsunriset.sh 2>&1"

# Embedded Perl script to calculate sunrise and sunset times
# Script requires: libdatetime-astro-sunrise-perl to be intsalled
# Result is in this format: 2015-01-08T08:41:00 to 2015-01-08T16:51:00
BOTH=$(/usr/bin/perl -x $0)
echo <<'__END__' > /dev/null
#! /usr/bin/perl -wl
# Change latitude and longitude to your location.

use DateTime;
use DateTime::Astro::Sunrise;
$latitude = "+51.592996"; $longitude = "+5.171443"; 
$sr = DateTime::Astro::Sunrise->new($longitude, $latitude, 0, 3);
$date = DateTime->now; $date->set_time_zone("local");
($rise, $set) = $sr->sunrise($date);
$rise->set_time_zone("local"); $set->set_time_zone("local");
print $rise, " to ", $set, "\n";
__END__
# End of embedded Perl script

# Isolate sunrise and sunset times
SUNRISE=$(echo "$BOTH" | awk '{ print $1 }' | awk -FT '{ print $2 }')
SUNSET=$(echo "$BOTH" | awk '{ print $3 }' | awk -FT '{ print $2 }')

# Split into hours and minutes
SUNRISE_H=$(echo "$SUNRISE" | awk -F: '{print $1}')
SUNRISE_H=$((10#$SUNRISE_H))    # Strip leading 0 (otherwise number is mistaken for octal base)
SUNRISE_M=$(echo "$SUNRISE" | awk -F: '{print $2}')
SUNRISE_M=$((10#$SUNRISE_M))    # Strip leading 0 (otherwise number is mistaken for octal base)
SUNSET_H=$(echo "$SUNSET" | awk -F: '{print $1}')
# No need to strip leading 0, there never is one in the sunset hour
SUNSET_M=$(echo "$SUNSET" | awk -F: '{print $2}')
SUNSET_M=$((10#$SUNSET_M))      # Strip leading 0 (otherwise number is mistaken for octal base)

# Add offset
SUNRISE_M=$((SUNRISE_M+OFFSET_RISE))
if [ $SUNRISE_M -gt 59 ]
then
	SUNRISE_M=$((SUNRISE_M-60))
	SUNRISE_H=$((SUNRISE_H+1))
fi
if [ $SUNRISE_M -lt 0 ]
then
	SUNRISE_M=$((SUNRISE_M+60))
	SUNRISE_H=$((SUNRISE_H-1))
fi

SUNSET_M=$((SUNSET_M+OFFSET_SET))
if [ $SUNSET_M -gt 59 ]
then
	SUNSET_M=$((SUNSET_M-60))
	SUNSET_H=$((SUNSET_H+1))
fi
if [ $SUNSET_M -lt 0 ]
then
	SUNSET_M=$((SUNSET_M+60))
	SUNSET_H=$((SUNSET_H-1))
fi

# Read current crontab
crontab -l > "$TEMP_FILE"

# See if the main cron task has already been installed
grep -q "$CRON_TSK_FLG" "$TEMP_FILE"
if [ $? -ne 0 ]
then
	# Not installed yet. Install it now
	echo -e "$CRON_TSK_FLG" >> "$TEMP_FILE"
	echo -e "$CRON_TSK_CMD\n" >> "$TEMP_FILE"
fi

# See if the sunrise and sunset scripts have already been installed
grep -q "$CRON_HEADER" "$TEMP_FILE"
if [ $? -eq 1 ]
then
	# Entries don't exist yet, add them
	echo -e "$CRON_HEADER" >> "$TEMP_FILE"
	echo -e "$SUNRISE_M\t$SUNRISE_H\t*\t*\t*\t$CRON_CMD_RISE\t\t$CRON_FLG_RISE" >> "$TEMP_FILE"
	echo -e "$SUNSET_M\t$SUNSET_H\t*\t*\t*\t$CRON_CMD_SET\t\t$CRON_FLG_SET" >> "$TEMP_FILE"
else
	# Entries do exist, change them
	sed -i "/$CRON_FLG_RISE/c$SUNRISE_M\t$SUNRISE_H\t*\t*\t*\t$CRON_CMD_RISE\t\t$CRON_FLG_RISE" "$TEMP_FILE"
	sed -i "/$CRON_FLG_SET/c$SUNSET_M\t$SUNSET_H\t*\t*\t*\t$CRON_CMD_SET\t\t$CRON_FLG_SET" "$TEMP_FILE"
fi

# Setup new crontab
crontab "$TEMP_FILE"

# Remove temp file
rm "$TEMP_FILE"
